;**************************************************************************
;Real Time Clock (RTC) erstmalig stellen und auslesen
;**************************************************************************

RTC_Neustart:
	ldi		tmp, (1<<TWINT | 1<<TWSTA | 1<<TWEN)
	out		TWCR, tmp					;Startbedingung ausgeben
	rcall	WarteAufRTC					;Warten bis TWINT = 0

	ldi		tempdat,TWIDispAdr + TWIwr
	out		TWDR, tempdat				;Slaveadresse "schreiben" laden
	ldi		tmp, (1<<TWINT | 1<<TWEN)
	out		TWCR, tmp					;Slaveadresse ausgeben
	rcall	WarteAufRTC	

	ldi 	tempdat,Sek_Register			
	out		TWDR, tempdat				;Registeradresse laden
	ldi		tmp, (1<<TWINT | 1<<TWEN)
	out		TWCR, tmp					;Registeradresse ausgeben
	rcall	WarteAufRTC	

	ldi 	tempdat, C_Start			
	out		TWDR, tempdat				;Registeradresse laden
	ldi		tmp, (1<<TWINT | 1<<TWEN)
	out		TWCR, tmp					;Registeradresse ausgeben
	rcall	WarteAufRTC	
	ret

;************************************************************************
RTC_Write:								;und Speicherung im SRAM
	ldi		tmp, (1<<TWINT | 1<<TWSTA | 1<<TWEN)
	out		TWCR, tmp					;Startbedingung ausgeben
	rcall	WarteAufRTC					;Warten bis TWINT = 0

	ldi		tempdat,TWIDispAdr + TWIwr
	out		TWDR, tempdat				;Slaveadresse "schreiben" laden
	ldi		tmp, (1<<TWINT) | (1<<TWEN)
	out		TWCR, tmp					;Slaveadresse ausgeben
	rcall	WarteAufRTC	

	ldi 	tempdat,Sek_Register			
	out		TWDR, tempdat				;Registeradresse laden
	ldi		tmp, (1<<TWINT) | (1<<TWEN)
	out		TWCR, tmp					;Registeradresse ausgeben
	rcall	WarteAufRTC	

	ldi 	tempdat, Sekunden			
	out		TWDR, tempdat				;Sekunden laden
	ldi		tmp, (1<<TWINT) | (1<<TWEN)
	out		TWCR, tmp					;Sekunden ausgeben
	rcall	WarteAufRTC	

	ldi 	tempdat, Minuten		
	out		TWDR, tempdat				;Minuten laden
	ldi		tmp, (1<<TWINT) | (1<<TWEN)
	out		TWCR, tmp					;Minuten ausgeben
	rcall	WarteAufRTC	

	ldi 	tempdat, Stunden			
	out		TWDR, tempdat				;Stunden laden
	ldi		tmp, (1<<TWINT) | (1<<TWEN)
	out		TWCR, tmp					;Stunden ausgeben
	rcall	WarteAufRTC	

	ldi 	tempdat, Wochentag		
	out		TWDR, tempdat				;Wochentag laden
	ldi		tmp, (1<<TWINT) | (1<<TWEN)
	out		TWCR, tmp					;Wochentag ausgeben
	rcall	WarteAufRTC	

	ldi 	tempdat, Datum		
	out		TWDR, tempdat				;Datum laden
	ldi		tmp, (1<<TWINT) | (1<<TWEN)
	out		TWCR, tmp					;Datum ausgeben
	rcall	WarteAufRTC	

	ldi 	tempdat, Monat			
	out		TWDR, tempdat				;Monat laden
	ldi		tmp, (1<<TWINT) | (1<<TWEN)
	out		TWCR, tmp					;Monat ausgeben
	rcall	WarteAufRTC
	
	ldi 	tempdat, Jahr			
	out		TWDR, tempdat				;Jahr laden
	ldi		tmp, (1<<TWINT) | (1<<TWEN)
	out		TWCR, tmp					;Jahr ausgeben
	rcall	WarteAufRTC	

	ldi 	tempdat, 0x80			
	out		TWDR, tempdat				;Kontrolldaten laden
	ldi		tmp, (1<<TWINT) | (1<<TWEN)
	out		TWCR, tmp					;Kontrolldaten ausgeben
	rcall	WarteAufRTC	

;STOP
	ldi		tmp, (1<<TWINT) | (1<<TWSTO) | (1<<TWEN)
	out		TWCR, tmp					;Stoppbedingung ausgeben
	ret

;*********************************************************************************************
;Auslesen der einzelnen RTC Werte,
;Umwandlung von BCD in binre Zehner und Einer 
;und Speicherung im SRAM
;*********************************************************************************************
RTC_Read:								;und Speicherung im SRAM
	ldi		tmp, (1<<TWINT | 1<<TWSTA | 1<<TWEN)
	out		TWCR, tmp					;Startbedingung ausgeben
	rcall	WarteAufRTC					;Warten bis TWINT = 0

	ldi		tempdat,TWIDispAdr + TWIwr
	out		TWDR, tempdat				;Slaveadresse "schreiben" laden
	ldi		tmp, (1<<TWINT | 1<<TWEN)
	out		TWCR, tmp					;Slaveadresse ausgeben
	rcall	WarteAufRTC	

	ldi 	tempdat, Sek_Register			
	out		TWDR, tempdat				;Registeradresse laden
	ldi		tmp, (1<<TWINT | 1<<TWEN)
	out		TWCR, tmp					;Registeradresse ausgeben
	rcall	WarteAufRTC	

;START lesen
	ldi		tmp, (1<<TWINT | 1<<TWSTA | 1<<TWEN)
	out		TWCR, tmp					;Startbedingung ausgeben
	rcall	WarteAufRTC					;Warten bis TWINT = 0

	ldi		tempdat,TWIDispAdr + TWIrd
	out		TWDR, tempdat				;Slaveadresse "lesen" laden
	ldi		tmp, (1<<TWINT | 1<<TWEN)
	out		TWCR, tmp					;Slaveadresse ausgeben
	rcall	WarteAufRTC
	
	ldi		tmp, (1<<TWINT | 1<<TWEA | 1<<TWEN) ;ACK fr mehrere Datenbytes senden
	out		TWCR, tmp
	rcall	WarteAufRTC

	ldi 	XL,low(Sek_Z)				;Zeiger auf SRAM-Location "Sek_Z"
	ldi 	XH,high(Sek_Z)

	ldi 	t_tmp,6						;Zhler bis Monat

Read_Loop:
	in		tempdat, TWDR				;Daten auslesen
	ldi		tmp, (1<<TWINT | 1<<TWEA | 1<<TWEN) ;ACK fr mehrere Datenbytes senden
	out		TWCR, tmp						
	rcall	WarteAufRTC	

	mov 	tmp,tempdat
	rcall	BCD_Aufl	
	dec 	t_tmp						;Register Monat gelesen?
	brne 	Read_Loop						

	in		tempdat, TWDR				;Jahr auslesen
	ldi		tmp, (1<<TWINT | 0<<TWEA | 1<<TWEN) ;NACK senden
	out		TWCR, tmp						
	rcall	WarteAufRTC	

	mov 	tmp,tempdat
	rcall	BCD_Aufl	

;STOP
	ldi		tmp, (1<<TWINT) | (1<<TWSTO) | (1<<TWEN)
	out		TWCR, tmp					;Stoppbedingung ausgeben

	ldi		RTC_Flag, 1
	ret

;***************************************************************************
WarteAufRTC:							;auf RTC Acknowledge warten
	in		tmp, TWCR					;Steuerregister lesen
	sbrs	tmp, TWINT
	rjmp	WarteAufRTC					;Warten bis TWINT = 0
	ret

;********************************************************************************
BCD_Aufl:								;Umwandlung BCD in Binr
	push	tmp
	rcall	BCD_loop
	pop		tmp
	swap	tmp

BCD_loop:
	andi	tmp, 0xF0
	swap	tmp
	st		X+, tmp						;Speicherung im SRAM
	ret	
